


; ********************************************************************
;  Basis-Routinen fuer das Interface Transmission Protokoll Version 2
; ********************************************************************







; Funktion: Schreiben der Anzahl der BOOT-Sektoren (SEND SECNT)
;
; Aufruf: INT_SE_SECNT
; Uebergabeparameter: Register B = Anzahl der BOOT-Sektoren 
; Rueckgabeparameter: keine
; 
INT_SE_SECNT:
		PUSH	BC
		
		; Umschalten des Interface Transmission Protokoll (ITP) auf Schreiben (WR)
		CALL	INT_MODE_WR
		;
		LD	B,SE_SECNT
		CALL	ITP_SEND_01
		;

		POP	BC
		CALL	ITP_SEND_01
		;

		; Umschalten des Interface Transmission Protokoll (ITP) auf Lesen (RD)
		CALL	INT_MODE_RD
		;
		RET




; Funktion: Lesen der Anzahl der BOOT-Sektoren (RECIEVE SECNT)
;
; Aufruf: INT_RE_SECNT
; Uebergabeparameter: keine
; Rueckgabeparameter: Register A = Anzahl der BOOT-Sektoren
; 
INT_RE_SECNT:
		; Umschalten des Interface Transmission Protokoll (ITP) auf Schreiben (WR)
		CALL	INT_MODE_WR
		;
		LD	B,RE_SECNT
		CALL	ITP_SEND_01
		;
		; Umschalten des Interface Transmission Protokoll (ITP) auf Lesen (RD)
		CALL	INT_MODE_RD
		;
		CALL	ITP_RECEIVE_01		; Abholen der Daten, Ergebnis in A
		;
		RET





; Funktion: Brennen des Inhaltes des RAM in den FLASH-Speicher auf der RAM-FLOPPY (SAVE FL2RAM) 
; Bemerkung: Es werden nur die 64K-Bloecke gebrannt, die auch vorher veraendert wurden
;
; Aufruf: INT_RD_SA_RFL
; Uebergabeparameter: keine
; Rueckgabeparameter: keine 
;    
INT_RD_SA_RFL:
		; Umschalten des Interface Transmission Protokoll (ITP) auf Schreiben (WR)
		CALL	INT_MODE_WR
		;
		LD	B,RE_SAVE_RAM2FL
		CALL	ITP_SEND_01
		;
		; Umschalten des Interface Transmission Protokoll (ITP) auf Lesen (RD)
		CALL	INT_MODE_RD
		;
		RET





; Funktion: Kopieren des FLASH-Speichers in den RAM auf der RAM-FLOPPY (COPY FL2RAM) 
;
; Aufruf: INT_RD_CP_RFL
; Uebergabeparameter: keine
; Rueckgabeparameter: keine 
;    
INT_RD_CP_RFL:
		; Umschalten des Interface Transmission Protokoll (ITP) auf Schreiben (WR)
		CALL	INT_MODE_WR
		;
		LD	B,RE_COPY_FL2RAM
		CALL	ITP_SEND_01
		;
		; Umschalten des Interface Transmission Protokoll (ITP) auf Lesen (RD)
		CALL	INT_MODE_RD
		;
		RET




; Funktion: Lesen des Fehlercodes (READ ERROR CODE) 
;
; Aufruf: INT_RD_ERROR
; Uebergabeparameter: keine
; Rueckgabeparameter: Register H = Fehler-Code
;                     Register L = Parameter 
;    
INT_RD_ERROR:
		; Umschalten des Interface Transmission Protokoll (ITP) auf Schreiben (WR)
		CALL	INT_MODE_WR
		;
		LD	B,RE_ERROR_CODE
		CALL	ITP_SEND_01
		;
		; Umschalten des Interface Transmission Protokoll (ITP) auf Lesen (RD)
		CALL	INT_MODE_RD
		;
		CALL	ITP_RECEIVE_01		; Abholen des Fehlercodes, Ergebnis in A
		LD	H,A
		PUSH	HL
		;
		CALL	ITP_RECEIVE_01		; Abholen des Parameters, Ergebnis in A
		POP	HL
		LD	L,A
		;
		RET






; Funktion: Lesen des Geraetecodes (READ DEVICE CODE) 
;
; Aufruf: INT_RD_DEV
; Uebergabeparameter: keine
; Rueckgabeparameter: Register A = WHO AM I - Code
;    
INT_RD_DEV:
		; Umschalten des Interface Transmission Protokoll (ITP) auf Schreiben (WR)
		CALL	INT_MODE_WR
		;
		LD	B,WHO_ARE_YOU
		CALL	ITP_SEND_01
		;
		; Umschalten des Interface Transmission Protokoll (ITP) auf Lesen (RD)
		CALL	INT_MODE_RD
		;
		CALL	ITP_RECEIVE_01		; Abholen der Daten, Ergebnis in A
		;
		RET





; Funktion: Setzen der Index- und Adressregister auf die Tabellen nach RAM_AKT_CON  
;
; Aufruf: INT_SET_ADR
; Uebergabeparameter: keine 
; Rueckgabeparameter: Register IX = D_ITP2_TAB + RAM_AKT_CON
;                              IY = C_ITP2_TAB + RAM_AKT_CON
;                              HL = RAM_CON_TAB + RAM_AKT_CON
INT_SET_ADR:
		;
		; Laden der Tabellenadressen
		LD	IX,D_ITP2_TAB
		LD	IY,C_ITP2_TAB
		LD	HL,RAM_CON_TAB
		;
		;
		LD	A,(RAM_AKT_CON)
		DEC	A
		JR	Z,INT_SET_ADR_B
		;
		LD	B,A
		;
INT_SET_ADR_A:
		INC	IX
		INC	IY
		INC	HL
		;
		DJNZ	INT_SET_ADR_A
		;
INT_SET_ADR_B:
		;
		RET




; Funktion: Initialisieren des Interface Transmission Protokoll (ITP) und Umschalten auf auf Lesen (RD)
;
; Aufruf: INT_INIT_RD
; Uebergabeparameter: keine
; Rueckgabeparameter: Register A=1 -> alles ok
;                     Register A=0 -> keine Reaktion der Schnittstelle auf die Umschaltung 
; 
INT_INIT_RD:	

		CALL	INT_SET_ADR


		; Ruecksetzen des DDWR-Bits und damit Anforderung
		; der Umschaltung auf Lesebetrieb (RD) 
		LD	A,(HL)
		RES	BIT_DDWR,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A
		;

		; Zur Kontrolle wir die rote LED eingeschaltet
		LD	A,(RAM_SYSREG)
		SET	BIT_LED_RT,A
		OUT	(D_SYS_PORT),A
		LD	(RAM_SYSREG),A
		;


		; Zeitschleife laden
		LD	A,00
		LD	(RAM_ZEIT_ZAE1),A
		;
		LD	A,00
		LD	(RAM_ZEIT_ZAE2),A
		;


		; nun wird auf die Rueckmeldung vom ATMEGA8 und von
		; der CONSOLE gewartet

INT_INIT_RD_A:
		; kleine Zeitschleife
		LD	A,(RAM_ZEIT_ZAE1)
		DEC	A
		LD	(RAM_ZEIT_ZAE1),A
		JR	NZ,INT_INIT_RD_B
		;
		; grosse Zeitschleife
		LD	A,(RAM_ZEIT_ZAE2)
		DEC	A
		LD	(RAM_ZEIT_ZAE2),A
		JR	Z,INT_INIT_RD_C
		;
INT_INIT_RD_B:
		;
		LD	C,(IX)
		IN	A,(C)
		BIT	BIT_DDAC,A
		JR	NZ,INT_INIT_RD_A	; warten, bis der ATMEGA8 und
						; die CONSOLE auf RD umgeschaltet haben
						; und dies durch Ruecksetzen der
						; Steuerleitung BIT_DDAC bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet


		; jetzt kann die rote LED wieder abgeschaltet werden
		LD	A,(RAM_SYSREG)
		RES	BIT_LED_RT,A
		RES	BIT_LED_GE,A
		OUT	(D_SYS_PORT),A
		LD	(RAM_SYSREG),A

		; Grundeinstellung der Steuerleitung SRAC
		LD	A,(HL)
		RES	BIT_SRAC,A
		RES	BIT_SDEN,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A
		;

		LD	C,(IY)
		; ITP2-Connector1 - MODE3 Bitbetrieb (Maske folgt)
		LD	A,0CFH
		OUT	(C),A

		; ITP2-Connector1 - Bitmaske - Read-Kommando ITP2
		LD	A,C_ITP2_RD
		OUT	(C),A


		; Grundeinstellung der Steuerleitung SRAC
		LD	A,(HL)
		LD	C,(IX)
		OUT	(C),A
		;
		LD	A,1
		;
		RET




INT_INIT_RD_C:
		LD	A,0
		;
		RET








; Funktion: Umschalten des Interface Transmission Protokoll (ITP) auf Lesen (RD)
;
; Aufruf: INT_MODE_RD
; Uebergabeparameter: keine
; Rueckgabeparameter: keine
; 
INT_MODE_RD:	
		CALL	INT_SET_ADR

		LD	A,(HL)
		BIT	BIT_DDWR,A
		RET	Z			; Ruecksprung, wenn RD bereits
						; aktiv ist (DDWR=0) 

		; Damit waehrend des Umschaltens nicht zwei Ausgaenge auf einer Leitung
		; aktiv sind, werden als Erstes alle Bidirektionalen Leitungen
		; als Eingang programmiert

		LD	C,(IY)
		; ITP2-Connector1 - MODE3 Bitbetrieb (Maske folgt)
		LD	A,0CFH
		OUT	(C),A

		; ITP2-Connector1 - Bitmaske - Initialwert-Kommandos ITP2
		LD	A,C_ITP2_IW
		OUT	(C),A
		;


		; Ruecksetzen des DDWR-Bits und damit Anforderung
		; der Umschaltung auf Lesebetrieb (RD) 
		LD	A,(HL)
		RES	BIT_DDWR,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A

		; Zur Kontrolle wir die rote LED eingeschaltet
		LD	A,(RAM_SYSREG)
		SET	BIT_LED_RT,A
		OUT	(D_SYS_PORT),A
		LD	(RAM_SYSREG),A


		; nun wird auf die Rueckmeldung vom ATMEGA8 und von
		; der CONSOLE gewartet
		;
		LD	C,(IX)
INT_MODE_RD_A:	
		IN	A,(C)
		BIT	BIT_DDAC,A
		JR	NZ,INT_MODE_RD_A	; warten, bis der ATMEGA8 und
						; die CONSOLE auf RD umgeschaltet haben
						; und dies durch Ruecksetzen der
						; Steuerleitung BIT_DDAC bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet

		; jetzt kann die rote LED wieder abgeschaltet werden
		LD	A,(RAM_SYSREG)
		RES	BIT_LED_RT,A
		RES	BIT_LED_GE,A
		OUT	(D_SYS_PORT),A
		LD	(RAM_SYSREG),A

		; Grundeinstellung der Steuerleitung SRAC
		LD	A,(HL)
		RES	BIT_SRAC,A
		RES	BIT_SDEN,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A
		;

		LD	C,(IY)
		; ITP2-Connector1 - MODE3 Bitbetrieb (Maske folgt)
		LD	A,0CFH
		OUT	(C),A

		; ITP2-Connector1 - Bitmaske - Read-Kommando ITP2
		LD	A,C_ITP2_RD
		OUT	(C),A


		; Grundeinstellung der Steuerleitung SRAC
		LD	A,(HL)
		LD	C,(IX)
		OUT	(C),A
		;

		RET




; Funktion: Umschalten des Interface Transmission Protokoll (ITP) auf Schreiben (WR)
;
; Aufruf: INT_MODE_WR
; Uebergabeparameter: keine
; Rueckgabeparameter: keine
; 
INT_MODE_WR:	
		CALL	INT_SET_ADR

		LD	A,(HL)
		BIT	BIT_DDWR,A
		RET	NZ			; Ruecksprung, wenn WR bereits
						; aktiv ist (DDWR=1) 


		; Damit waehrend des Umschaltens nicht zwei Ausgaenge auf einer Leitung
		; aktiv sind, werden als Erstes alle Bidirektionalen Leitungen
		; als Eingang programmiert

		LD	C,(IY)
		; ITP2-Connector1 - MODE3 Bitbetrieb (Maske folgt)
		LD	A,0CFH
		OUT	(C),A

		; ITP2-Connector1 - Bitmaske - Initialwert-Kommandos ITP2
		LD	A,C_ITP2_IW
		OUT	(C),A
		;


		; Setzen des DDWR-Bits und damit Anforderung
		; der Umschaltung auf Schreibbetrieb (WR) 
		LD	A,(HL)
		SET	BIT_DDWR,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A
		;


		; Zur Kontrolle wir die rote LED eingeschaltet
		LD	A,(RAM_SYSREG)
		SET	BIT_LED_RT,A
		OUT	(D_SYS_PORT),A
		LD	(RAM_SYSREG),A
		;


		; nun wird auf die Rueckmeldung vom ATMEGA8 und von
		; der CONSOLE gewartet
		;
		LD	C,(IX)
INT_MODE_WR_A:	
		IN	A,(C)
		BIT	BIT_DDAC,A
		JR	Z,INT_MODE_WR_A		; warten, bis der ATMEGA8 und
						; die CONSOLE auf WR umgeschaltet haben
						; und dies durch Setzen der
						; Steuerleitung BIT_DDAC bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet


		; jetzt kann die rote LED wieder abgeschaltet werden
		LD	A,(RAM_SYSREG)
		RES	BIT_LED_RT,A
		SET	BIT_LED_GE,A
		OUT	(D_SYS_PORT),A
		LD	(RAM_SYSREG),A


		; Grundeinstellung der Steuerleitung SDEN
		LD	A,(HL)
		RES	BIT_SRAC,A
		RES	BIT_SDEN,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A
		;

		LD	C,(IY)
		; ITP2-Connector1 - MODE3 Bitbetrieb (Maske folgt)
		LD	A,0CFH
		OUT	(C),A

		; ITP2-Connector1 - Bitmaske - Write-Kommando ITP2
		LD	A,C_ITP2_WR
		OUT	(C),A


		; Grundeinstellung der Steuerleitung SDEN
		LD	A,(HL)
		LD	C,(IX)
		OUT	(C),A
		;

		RET




; Funktion: ITP - RECEIVE 01
;
; Aufruf: ITP_RECEIVE_01
; Uebergabeparameter: keine
; Rueckgabeparameter: Register A, das empfangene Byte
; 
ITP_RECEIVE_01:
		CALL	INT_SET_ADR

		CALL	ITP_RECEIVE_01X
		PUSH	AF
		CALL	ITP_RECEIVE_01X
		POP	BC
		;
		; Low-Teil in B, High-Teil in A
		;
		SLA	A
		SLA	A
		SLA	A
		SLA	A
		;
		ADD	A,B
		;
		RET



ITP_RECEIVE_01X:

		; Zur Kontrolle wir die rote LED eingeschaltet
		LD	A,(RAM_SYSREG)
		SET	BIT_LED_RT,A
		OUT	(D_SYS_PORT),A
		LD	(RAM_SYSREG),A
		;

		; pauschales Ruecksetzen von SRAC
		LD	A,(HL)
		RES	BIT_SRAC,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A
		;

ITP_RECEIVE_01A:	
		IN	A,(C)
		BIT	BIT_SDEN,A
		JR	Z,ITP_RECEIVE_01A	; warten, bis der ATMEGA8 das Datenbyte bereit
						; gestellt hat und dies durch Setzen der
						; Steuerleitung BIT_SDEN bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet
		;

		; etwas warten, bis die Pegel sicher stehen
		NOP
		NOP
		NOP
		NOP

		; jetzt koennen die Daten aus PIO 1 Port A gelesen werden
		IN	A,(C)
		LD	B,A
		;
		LD	A,(HL)
		SET	BIT_SRAC,A
		OUT	(C),A
		LD	(HL),A
		;
ITP_RECEIVE_01B:	
		IN	A,(C)
		BIT	BIT_SDEN,A
		JR	NZ,ITP_RECEIVE_01B	; warten, bis der ATMEGA8 erkannt hat,
						; dass die Daten uebernommen wurden
						; und dies durch Ruecksetzen der
						; Steuerleitung BIT_SDEN bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet


		; jetzt kann die rote LED wieder abgeschaltet werden
		LD	A,(RAM_SYSREG)
		RES	BIT_LED_RT,A
		OUT	(D_SYS_PORT),A
		LD	(RAM_SYSREG),A

		; Grundeinstellung der Steuerleitung SRAC
		LD	A,(HL)
		RES	BIT_SRAC,A
		RES	BIT_SDEN,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A
		;

		; Rckgabe des Datenbytes (D0..D3) in Register A
		LD	A,B
		AND	00001111B
		;
		RET
	


; Funktion: ITP - SEND 01
;
; Aufruf: ITP_SEND_01
; Uebergabeparameter: Register B - das zu sendende Byte
; Rueckgabeparameter: keine
; 
ITP_SEND_01:
		PUSH	BC
		CALL	INT_SET_ADR
		POP	BC

		PUSH	BC
		;
		LD	A,B
		AND	00001111B
		LD	B,A
		;
		CALL	ITP_SEND_01X
		;
		POP	BC
		;
		SRL	B
		SRL	B
		SRL	B
		SRL	B
		;
		CALL	ITP_SEND_01X
		;
		RET



ITP_SEND_01X:

		; Zur Kontrolle wir die rote LED eingeschaltet
		LD	A,(RAM_SYSREG)
		SET	BIT_LED_RT,A
		OUT	(D_SYS_PORT),A
		LD	(RAM_SYSREG),A
		;

		; pauschales Ruecksetzen von SDEN
		LD	A,(HL)
		RES	BIT_SDEN,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A
		;

		;
ITP_SEND_01A:	IN	A,(C)
		BIT	BIT_SRAC,A
		JR	NZ,ITP_SEND_01A		; warten, bis der ATMEGA8 und
						; die CONSOLE den Grundzustand eingestellt
						; haben und dies durch Ruecksetzen der
						; Steuerleitung BIT_SRAC bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet
		;
		LD	A,(HL)
		AND	11110000B
		;
		OR	B			; zusammenschalten der Daten- und Steuerbits
						; in Register A
		OUT	(C),A			; Daten an Port senden
		;
		NOP
		NOP
		NOP
		;
		; Setzen von SDEN
		SET	BIT_SDEN,A
		OUT	(C),A
		LD	(HL),A
		;
		NOP
		NOP
		;
ITP_SEND_01_B:	IN	A,(C)
		BIT	BIT_SRAC,A
		JR	Z,ITP_SEND_01_B		; warten, bis der ATMEGA8 und
						; die CONSOLE die Daten uebernommen haben
						; und dies durch Setzen der
						; Steuerleitung BIT_SRAC bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet
		;

		; jetzt kann die rote LED wieder abgeschaltet werden
		LD	A,(RAM_SYSREG)
		RES	BIT_LED_RT,A
		OUT	(D_SYS_PORT),A
		LD	(RAM_SYSREG),A

		; Rcksetzen von SDEN
		LD	A,(HL)
		RES	BIT_SRAC,A
		RES	BIT_SDEN,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A
		;
		RET
	


