


; ********************************************************************
;  Initialisierung des Interface Transmission Protokoll Version 2
; ********************************************************************




; Funktion: 300 ms Warten, bis die Hardware an den Connectoren initialisiert ist
;
; Aufruf: WAITINIT
; Uebergabeparameter: keine
; Rueckgabeparameter: keine
; 
WAITINIT:
		;
		LD	B,3
WAITINIT_A:
		PUSH	BC
		CALL	TOGGLE_LED_RT
		LD	A,1
		CALL	WAIT_01S
		POP	BC
		;
		DJNZ	WAITINIT_A
		;
		RET



; Funktion: Ermittlung von angeschlossenen Connectoren
;
;           Dazu werden die PIOs, die als Connector dienen sollen (PIO1B...PIO3B) als Erstes
;           als Eingaenge initialisiert und dann die Pegel eingelesen. Ist ein Gerat an eienem
;           Connector angeschlossen, so werden die Pegel des ITP2 (die durch Widerstaende eingestellt
;           werden) eingelesen und ausgewertet.
;
; Aufruf: INITCONX
; Uebergabeparameter: keine
; Rueckgabeparameter: keine
; 
INITCONX:

		; Setzen der RAM-Zellen fuer die ITP2-Schnittstelle
		; ---------------------------------------------------

		; 
		LD 	A,D_ITP2_IW
		LD	HL,RAM_CON_TAB
		;
		LD	(HL),A
		INC	HL
		LD	(HL),A
		INC	HL
		LD	(HL),A
		INC	HL
		LD	(HL),A
		INC	HL
		LD	(HL),A
		;


		; 1. Lauf Initialisierung der Connectoren auf Eingang
		; ---------------------------------------------------

		; Zaehler fuer den initialisierten Connector
		LD	B,5
		;
		; Tabelle fuer Portadressen der Kommandos
		LD	HL,C_ITP2_TAB
		;
INITCONX_D:
		LD	C,(HL)
		;
		; MODE3 Bitbetrieb (Maske folgt)
		LD	A,0CFH
		OUT	(C),A
		; 
		; alles Eingaenge
		LD	A,BITMSKAE
		OUT	(C),A
		;
		INC	HL
		;
		DJNZ	INITCONX_D
		;


		; 2. Lauf nur einfach Lesen der Daten und Abspeichern
		; ---------------------------------------------------

		; Zaehler fuer den getesteten Connector
		LD	B,5
		;
		; Tabelle fuer Portadressen der Daten
		LD	HL,D_ITP2_TAB
		;
		; Tabelle fuer Ergebnisse
		LD	DE,RAM_ITP2_IWX
		;
INITCONX_A:
		; Laden der aktuellen Portadesse-Daten in Reg C
		LD	C,(HL)
		; Laden des Datenwertes des Ports in Reg A
		IN	A,(C)
		; Abspeichern des Datenwertes in die Tabelle
		LD	(DE),A
		;
		INC	HL		; naechste Portadresse-Daten
		INC	DE		; naechster Tabellenplatz  
		;
		DJNZ	INITCONX_A
		;

		; 3. Lauf Auswerten der gelesenen Daten
		; ---------------------------------------------------

		;
		; Tabelle fuer Ergebnisse
		LD	HL,RAM_ITP2_IWX
		;
		; letzter Eintrag
		INC	HL
		INC	HL
		INC	HL
		INC	HL

		; Laden der Speicherzelle fuer den aktuellen Connector
		; mit einem Standardwert -> 1
		LD	A,1
		LD	(RAM_AKT_CON),A
		;
		; Zaehler fuer den getesteten Connector
		LD	B,5
		;
		LD	C,0
INITCONX_B:
		LD	A,(HL)
		LD	(HL),C		; Standardbelegung = 0 (kein Connector)

		AND	11110000B	; Ausblenden der Datenleitungen,
					; es interessieren nur die Steuerleitungen

		CP	00110000B	; DDWR und DDAC werden mit Widerstaenden auf 1 gelegt
					; SDEN und SRAC werden mit Widerstaenden auf 0 gelegt
		;
		JR	NZ,INITCONX_C
		;
		LD	A,01H
		LD	(HL),A		; ist Connector vorhanden, so ist der Eintrag = 1
		;
		LD	A,B
		LD	(RAM_AKT_CON),A	; wurden die Pegel des ITP2 erkannt, so ist ein
					; Connector vorhanden und die Portnummer des niedrigsten
					; Connectors (ab 1) wir in RAM_AKT_CON abgespeichert 
		;
INITCONX_C:
		DEC	HL	; naechster Tabellenwert (von unten nach oben)
		;
		DJNZ	INITCONX_B
		;
		RET




; Funktion: Pruefung der angeschlossenen Connectoren
;
;
; Aufruf: READCONX
; Uebergabeparameter: keine
; Rueckgabeparameter: keine
; 
READCONX:
		; Laden der Speicherzelle fuer den aktuellen Connector
		; mit einem Standardwert -> 1
		LD	A,1
		LD	(RAM_AKT_CON),A
		;
		; Tabelle fuer Ergebnisse
		LD	HL,RAM_ITP2_IWX
		;
		; letzter Eintrag
		INC	HL
		INC	HL
		INC	HL
		INC	HL

		; Zaehler fuer den getesteten Connector
		LD	B,5
		;
		
READCONX_A:
		LD	A,(HL)
		CP	0
		JR	Z,READCONX_B
		;
		LD	A,B
		LD	(RAM_AKT_CON),A
		;
		PUSH	BC
		PUSH	HL
		CALL	INT_INIT_RD
		POP	HL
		POP	BC
		;
		; wenn die Schnittstelle sauber geantwortet hat, dann bleibt
		; der aktive Eintrag in der Tabelle erhalten
		CP	1
		JR	Z,READCONX_B
		;
		; ansonsten wird er geloescht
		XOR	A
		LD	(HL),A
READCONX_B:
		DEC	HL
		;
		DJNZ	READCONX_A
		;
		RET




; Funktion: Auslesen der Funktion der angeschlossenen Geraete (Read connected devices)
;
; Aufruf: READ_CON_DEV
; Uebergabeparameter: keine
; Rueckgabeparameter: keine
; 
READ_CON_DEV:

		; Tabelle fuer Ergebnisse
		LD	HL,RAM_ITP2_IWX
		;
		; letzter Eintrag
		INC	HL
		INC	HL
		INC	HL
		INC	HL

		; Zaehler fuer den getesteten Connector
		LD	B,5
		;
		
READ_CON_DEV_A:
		LD	A,(HL)
		CP	0
		JR	Z,READ_CON_DEV_B
		;
		LD	A,B
		LD	(RAM_AKT_CON),A
		;
		PUSH	BC
		PUSH	HL
		CALL	INT_RD_DEV
		POP	HL
		POP	BC
		;
		LD	(HL),A
		;
READ_CON_DEV_B:
		DEC	HL
		;
		DJNZ	READ_CON_DEV_A
		;
		RET





; Funktion: Zuordnung der CP/M Schnittstellen zu den angeschlossenen Geraeten
;
; Aufruf: DEV_CONNECT
; Uebergabeparameter: keine
; Rueckgabeparameter: keine
; 
DEV_CONNECT:
		; Voreinstellen der Device-Connectoren
		XOR	A
		LD	(RAM_CON_CONOUT),A
		LD	(RAM_CON_CONOUT2),A
		LD	(RAM_CON_CONIN),A
		LD	(RAM_CON_DISK),A
		LD	(RAM_CON_DISK2),A
		LD	(RAM_CON_BOOT),A
		LD	(RAM_CON_LIST),A
		;
		; Tabelle fuer Ergebnisse
		LD	HL,RAM_ITP2_IWX
		;
		; letzter Eintrag
		INC	HL
		INC	HL
		INC	HL
		INC	HL

		; Zaehler fuer den getesteten Connector
		LD	B,5
		;
		
DEV_CONNECT_A:
		LD	A,(HL)
		CP	0
		JR	Z,DEV_CONNECT_X
		;
		CP	DEVCODE_PC_CPM_EMU
		JR	NZ,DEV_CONNECT_C
		;
		LD	A,(RAM_CON_CONOUT)			
		CP	0
		JR	Z,DEV_CONNECT_A2
		;
		LD	(RAM_CON_CONOUT2),A
		;
DEV_CONNECT_A2:
		LD	A,B
		LD	(RAM_CON_CONOUT),A		
		;
DEV_CONNECT_A3:
		LD	A,(RAM_CON_DISK)
		CP	0
		JR	Z,DEV_CONNECT_A4
		;
		LD	(RAM_CON_DISK2),A
		;
DEV_CONNECT_A4:
		LD	A,B
		LD	(RAM_CON_DISK),A		
		;
DEV_CONNECT_A5:
		LD	A,B
		LD	(RAM_CON_CONIN),A
		LD	(RAM_CON_BOOT),A
		LD	(RAM_CON_LIST),A
		;
		JR	DEV_CONNECT_X
		;

DEV_CONNECT_C:
		CP	DEVCODE_KEYBOARD
		JR	NZ,DEV_CONNECT_D
		;
		LD	A,B
		LD	(RAM_CON_CONIN),A
		;
		JR	DEV_CONNECT_X
		;

DEV_CONNECT_D:
		CP	DEVCODE_DISPLAY
		JR	NZ,DEV_CONNECT_E
		;
		LD	A,(RAM_CON_CONOUT)			
		CP	0
		JR	NZ,DEV_CONNECT_D2
		;
		LD	A,B
		LD	(RAM_CON_CONOUT),A
		JR	DEV_CONNECT_X		
		;
DEV_CONNECT_D2:
		LD	A,B
		LD	(RAM_CON_CONOUT2),A		
		;
		JR	DEV_CONNECT_X
		;

DEV_CONNECT_E:
		CP	DEVCODE_RAMFLOPPY
		JR	NZ,DEV_CONNECT_X
		;
		LD	A,(RAM_CON_DISK)
		CP	0
		JR	Z,DEV_CONNECT_E2
		;
		LD	(RAM_CON_DISK2),A
		;
DEV_CONNECT_E2:
		LD	A,B
		LD	(RAM_CON_DISK),A		
		;
DEV_CONNECT_E3:
		LD	A,B
		LD	(RAM_CON_BOOT),A
		;
		LD	(RAM_AKT_CON),A
		;
		PUSH	AF
		PUSH	BC
		PUSH	HL
		CALL	INT_RD_CP_RFL
		POP	HL
		POP	BC
		POP	AF
		;

DEV_CONNECT_X:
		DEC	HL
		;
		DJNZ	DEV_CONNECT_A
		;
		RET



